///////////////////////////////////////////////
// Y2KTEST.CPP                               //
// Copyright 1998, Phillip Hosey             //
// Licensed for use by EMS Consultants, LTD  //
///////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// STANDARD INCLUDES 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <graph.h> 
#include <dos.h>                             

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// A class for time and date functions 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class Time {
   private:
      unsigned char currentHour, currentMin, currentSec;
      unsigned char currentMonth, currentDay, currentCent, currentYear;

      unsigned char day;
      unsigned char month;
      unsigned char year;
      unsigned char century;
      
      unsigned char hours;
      unsigned char minutes;
      unsigned char seconds;

   public:
      unsigned char GetHour();  
      unsigned char GetCent();
      void GetCurrentTime();
      void GetCurrentDate();
      void SetCurrentTime();
      void SetCurrentDate();
      void GetBIOSDate();         
      void GetRTCDate();          
      void ShowDate();
      void GetBIOSTime();
      void GetRTCTime();
      void ShowTime();
      void RecordDate();
      void ReadDate();
};                            

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTION DEFINITIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned char Time::GetHour()
{
   return hours;
}

unsigned char Time::GetCent()
{
   return century;
}


void Time::GetCurrentTime()
{
   // Reads the current time fromt the BIOS
   
   unsigned char hour, min, sec;
   _asm
   {
      mov ah, 0x02
      int 0x1A
      mov hour,   ch
      mov min, cl
      mov sec, dh
   }       
   
   currentHour = hour;
   currentMin = min;
   currentSec = sec;
}

void Time::GetCurrentDate()
{
   // Reads the current date from the BIOS
   
   unsigned char dd, mm, yy, c;

   _asm
   {
      mov ah, 0x04
      int 0x1A
      mov dd, dl
      mov mm, dh
      mov yy, cl
      mov c, ch
   }           
   
   currentDay   = dd;
   currentMonth = mm;
   currentYear  = yy;
   currentCent = c;
}                       

void Time::GetBIOSDate()
{
   unsigned char dd, mm, yy, c;

   _asm
   {
      mov ah, 0x04
      int 0x1A
      mov dd, dl
      mov mm, dh
      mov yy, cl
      mov c, ch
   }           
   
   day=dd;
   month=mm;
   year=yy;
   century=c;
}                       

void Time::GetRTCDate()
{
   // Reads the date directly from the Real Time Clock
   
   unsigned char curMon, curDay, curYear, curCent;
   
   _asm
   {
      mov   al, 0x08     // the month register
      out   0x70, al
      in    al, 0x71
      mov   curMon, al
      
      mov   al, 0x07     // the day register
      out   0x70, al
      in    al, 0x71
      mov   curDay, al
      
      mov   al, 0x09     // the year register
      out   0x70, al
      in    al, 0x71
      mov   curYear, al  
      
      mov   al, 0x32     // the century register
      out   0x70, al
      in    al, 0x71
      mov   curCent, al
   }
   
   month   = curMon;
   day     = curDay;
   year    = curYear;
   century = curCent;
}
   


void Time::ShowDate()
{                        
  printf(" %02X/%02X/%02X%02X ", month, day, century, year);
}


void Time::GetBIOSTime()
{
   // Reads the time using BIOS routine
   
   unsigned char hour, minute, second=0;
   
   _asm
   {
      mov ah, 0x02
      int 0x1A
      mov hour,   ch
      mov minute, cl
      mov second, dh
   }
   
   hours = hour;
   minutes = minute;
   seconds = second;
   
}
   
void Time::GetRTCTime()                
{
   // Reads time directly from Real Time Clock
   
   unsigned char curHour, curMin, curSec;
   
   _asm
   {
      mov   al,      0x04        // Hour register
      out   0x70,    al
      in    al,      0x71
      mov   curHour, al    
      
      mov   al,      0x02        // Minute register
      out   0x70,    al
      in    al,      0x71
      mov   curMin,  al
      
      mov   al,      0x00        // Second register
      out   0x70,    al
      in    al,      0x71
      mov   curSec,  al
    }
    
    hours    = curHour;
    minutes  = curMin;
    seconds  = curSec;
}
      
   
void Time::ShowTime()
{
   if (hours == 0x0)
      printf(" 12:%02X:%02X AM\n ",minutes, seconds);
   if ((hours < 0x12) && (hours > 0x0))
      printf(" %02X:%02X:%02X AM\n ", hours, minutes, seconds);
   if (hours > 0x11)
      printf(" %02X:%02X:%02X PM\n ", hours, minutes, seconds);

}

void setTestTime(unsigned char hour, unsigned char minute, unsigned char second)
{
      
   // Sets up the time for the rollover and reboot tests.
   
   _asm
   {
      mov al, 0x04
      out 0x70, al
      mov al, hour
      out 0x71, al
   
      mov al, 0x02
      out 0x70, al
      mov al, minute
      out 0x71, al
      
      mov al, 0x00
      out 0x70, al
      mov al, second
      out 0x71, al
   }
}   

void setTestDate(unsigned char month, unsigned char day, 
                 unsigned char year, unsigned char cent)
{
   // Sets up the date for the rollover and reboot tests
      
   _asm
   {
      mov al, 0x08
      out 0x70, al
      mov al, month
      out 0x71, al
      
      mov al, 0x07
      out 0x70, al
      mov al, day
      out 0x71, al
      
      mov al, 0x32
      out 0x70, al
      mov al, cent
      out 0x71, al
      
      mov al, 0x09
      out 0x70, al
      mov al, year
      out 0x71, al
   }
}
       
void Time::SetCurrentTime()
{
   unsigned char hour, minute, second;
   
   hour   = currentHour;
   minute = currentMin;
   second = currentSec;
   
   _asm
   {
      mov al, 0x04
      out 0x70, al
      mov al, hour
      out 0x71, al
   
      mov al, 0x02
      out 0x70, al
      mov al, minute
      out 0x71, al
      
      mov al, 0x00
      out 0x70, al
      mov al, second
      out 0x71, al
   }           
   hours   = hour;
   minutes = minute;
   seconds = second;
}   

void Time::SetCurrentDate()
{
   unsigned char mm, dd, c, yy;
   
   mm = currentMonth;
   dd   = currentDay;
   c  = currentCent;
   yy  = currentYear;
   
   _asm
   {
      mov al, 0x08
      out 0x70, al
      mov al, mm
      out 0x71, al
      
      mov al, 0x07
      out 0x70, al
      mov al, dd
      out 0x71, al
      
      mov al, 0x32
      out 0x70, al
      mov al, c
      out 0x71, al
      
      mov al, 0x09
      out 0x70, al
      mov al, yy
      out 0x71, al
   }
   
   month   = mm;
   day     = dd;
   year    = yy;
   century = c;
}                  

void Time::RecordDate()
{
   // Records the current date and time to a file to be retrieved when the test is done.
   
   FILE *date = fopen("date.dat", "r+");
   
   int *buffer;
   
   buffer[1] = currentMonth;
   buffer[2] = currentDay;
   buffer[3] = currentYear;
   buffer[4] = currentCent;
   buffer[5] = currentHour;
   buffer[6] = currentMin;
   buffer[7] = currentSec;
   
   fwrite(buffer, sizeof(int), 7, date);
   fclose(date);
}

void Time::ReadDate()   
{
   // reads date and time from file
   
   FILE *date = fopen("date.dat", "r");
   
   int *buffer;        
   
   fread(buffer, sizeof(int), 7, date);
   
   currentMonth = buffer[1];
   currentDay   = buffer[2];
   currentYear  = buffer[3];
   currentCent  = buffer[4];
   currentHour  = buffer[5];
   currentMin   = buffer[6];
   currentSec   = buffer[7];
   
   fclose(date);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MAIN
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void main()
{
   // The first thing we do is check to see where we should be in the program, that is,
   // is it at the beginning, or are we running the 2nd test after having rebooted.
   
   char c; 
   Time now;

   FILE *fhandle = fopen("Y2KTEST.DAT", "r+");

   
   // If we have rebooted and are about to do the second test......
   if (fgetc(fhandle) == '2')
   {
      _clearscreen(_GCLEARSCREEN);
      
      now.GetBIOSDate();
      now.GetBIOSTime();
      
      if (now.GetCent() == 0x20)
      {
         printf("\n\nDate reported as ");
         now.ShowDate();
         printf("\n\nYour system is capable of holding dates beyond the year 2000");
      }
      else
      {
         printf("\n\nDate reported as ");
         now.ShowDate();
         printf("\nYour system is NOT capable of holding dates beyond the year 2000");
      }
      
      c = '1';
      fseek(fhandle, 0L, SEEK_SET);
      fputc(c, fhandle);
      
      now.ReadDate();
      now.SetCurrentDate();
      now.SetCurrentTime();
      now.GetBIOSDate();
      now.GetBIOSTime();

      _settextposition(10, 0);
      printf("\nCurrent Date reset to: ");
      now.ShowDate();
      printf("\nCurrent Time reset to: ");
      now.ShowTime();    
      printf("\n\nPlease reboot your system to insure that the operating");
      printf("\nsystem correctly receives the reset date and time.");
      _settextposition(24, 0);
      _settextcursor(0x0707);
      exit(0);
   }

   
   // This is where execution begins the first time we run the program.......
   _clearscreen(_GCLEARSCREEN); 
   _settextcursor(0x2000);

   now.GetCurrentDate();
   now.GetCurrentTime();
   now.RecordDate();

   now.GetBIOSTime();
   now.GetBIOSDate();

   _settextposition(1,15);
   printf("Y2K TESTER - v1.0 Copyright 1998 by Phillip Hosey");
   _settextposition(3, 20);
   printf("Licensed for use by EMS Consultants, LTD.");
   _settextposition(5, 9);
   printf("This test will determine if your system's BIOS is capable of");
   _settextposition(6, 9);
   printf("making the transition from 1999 to 2000 while it is running.");
   _settextposition(8, 22);
   printf("Press any key to start the test....");
   _settextposition(11, 0);
   printf(" BIOS Real-Time Rollover Test: ");

   getch();

   setTestDate(0x12, 0x31, 0x99, 0x19);
   setTestTime(0x23, 0x59, 0x50);
   now.GetBIOSTime();

   while (now.GetHour()!= 0x00)
   {
      now.GetBIOSDate();
      now.GetBIOSTime();
      _settextposition(11, 34);
      now.ShowDate();
      now.ShowTime();      
   }
   
   if (now.GetCent() != 0x20)
   {
      printf("\n\nYour system FAILED the Realtime BIOS rollover test");
   }
   else
   {
      printf("\n\nYour system PASSED the Realtime BIOS rollover test.");
   }   
   
   printf("\nPress any key to begin the next test...");
   getch();
   
   _clearscreen(_GCLEARSCREEN);
   printf("The next test will set your clock to 2-29-2000, and then reboot\n");
   printf("your system and test to see if the clock can hold that date.\n\n");
   printf("Press any key to reboot...");
   
   getch();
   
   setTestDate(0x2, 0x29, 0x00, 0x20);
   now.GetBIOSDate();
   now.GetBIOSTime();
   
  // set up the flag to tell the program the next time we run it
  // that we will be doing the 2nd test.
   c = '2';
   fseek(fhandle, 0L, SEEK_SET);
   fputc(c, fhandle);
   fclose(fhandle);

   // do reboot here 
   unsigned int far* reboot = (unsigned int far*)0x00400072;
   unsigned char far* address = (unsigned char far*)0xFFFF0000;
   
   *reboot = 0x00;
 
   _asm 
   {
      jmp address
      iret
   }
}
     

